@@ -7,11 +7,10 @@ import (
 	"runtime/debug"
 
 	"github.com/alecthomas/kong"
-	"github.com/charmbracelet/huh"
+	tea "github.com/charmbracelet/bubbletea"
+	"github.com/charmbracelet/gum/internal/exit"
 	"github.com/charmbracelet/lipgloss"
 	"github.com/muesli/termenv"
-
-	"github.com/charmbracelet/gum/internal/exit"
 )
 
 const shaLen = 7
@@ -55,6 +54,7 @@ func main() {
 		}),
 		kong.Vars{
 			"version":                 version,
+			"versionNumber":           Version,
 			"defaultHeight":           "0",
 			"defaultWidth":            "0",
 			"defaultAlign":            "left",
@@ -73,10 +73,18 @@ func main() {
 		},
 	)
 	if err := ctx.Run(); err != nil {
-		if errors.Is(err, exit.ErrAborted) || errors.Is(err, huh.ErrUserAborted) {
+		var ex exit.ErrExit
+		if errors.As(err, &ex) {
+			os.Exit(int(ex))
+		}
+		if errors.Is(err, tea.ErrProgramKilled) {
+			fmt.Fprintln(os.Stderr, "timed out")
+			os.Exit(exit.StatusTimeout)
+		}
+		if errors.Is(err, tea.ErrInterrupted) {
 			os.Exit(exit.StatusAborted)
 		}
-		fmt.Println(err)
+		fmt.Fprintln(os.Stderr, err)
 		os.Exit(1)
 	}
 }
