@@ -19,14 +19,11 @@ import (
 	"os"
 	"os/exec"
 	"strings"
-	"time"
-
-	"github.com/charmbracelet/gum/internal/exit"
-	"github.com/charmbracelet/gum/timeout"
-	"github.com/charmbracelet/x/term"
+	"syscall"
 
 	"github.com/charmbracelet/bubbles/spinner"
 	tea "github.com/charmbracelet/bubbletea"
+	"github.com/charmbracelet/x/term"
 )
 
 type model struct {
@@ -35,21 +32,22 @@ type model struct {
 	align      string
 	command    []string
 	quitting   bool
-	aborted    bool
+	isTTY      bool
 	status     int
 	stdout     string
 	stderr     string
 	output     string
-	showOutput bool
+	showStdout bool
+	showStderr bool
 	showError  bool
-	timeout    time.Duration
-	hasTimeout bool
 }
 
 var (
 	bothbuf strings.Builder
 	outbuf  strings.Builder
 	errbuf  strings.Builder
+
+	executing *exec.Cmd
 )
 
 type finishCommandMsg struct {
@@ -65,22 +63,18 @@ func commandStart(command []string) tea.Cmd {
 		if len(command) > 1 {
 			args = command[1:]
 		}
-		cmd := exec.Command(command[0], args...) //nolint:gosec
 
+		executing = exec.Command(command[0], args...) //nolint:gosec
 		if term.IsTerminal(os.Stdout.Fd()) {
-			stdout := io.MultiWriter(&bothbuf, &errbuf)
-			stderr := io.MultiWriter(&bothbuf, &outbuf)
-
-			cmd.Stdout = stdout
-			cmd.Stderr = stderr
+			executing.Stdout = io.MultiWriter(&bothbuf, &outbuf)
+			executing.Stderr = io.MultiWriter(&bothbuf, &errbuf)
 		} else {
-			cmd.Stdout = os.Stdout
+			executing.Stdout = os.Stdout
+			executing.Stderr = os.Stderr
 		}
-
-		_ = cmd.Run()
-
-		status := cmd.ProcessState.ExitCode()
-
+		executing.Stdin = os.Stdin
+		_ = executing.Run()
+		status := executing.ProcessState.ExitCode()
 		if status == -1 {
 			status = 1
 		}
@@ -94,48 +88,49 @@ func commandStart(command []string) tea.Cmd {
 	}
 }
 
+func commandAbort() tea.Msg {
+	if executing != nil && executing.Process != nil {
+		_ = executing.Process.Signal(syscall.SIGINT)
+	}
+	return tea.InterruptMsg{}
+}
+
 func (m model) Init() tea.Cmd {
 	return tea.Batch(
 		m.spinner.Tick,
 		commandStart(m.command),
-		timeout.Init(m.timeout, nil),
 	)
 }
 
 func (m model) View() string {
-	if m.quitting && m.showOutput {
-		return strings.TrimPrefix(errbuf.String()+"\n"+outbuf.String(), "\n")
+	var out string
+	if m.showStderr {
+		out += errbuf.String()
+	}
+	if m.showStdout {
+		out += outbuf.String()
 	}
 
-	var str string
-	if m.hasTimeout {
-		str = timeout.Str(m.timeout)
+	if m.quitting {
+		return out
 	}
+
+	if !m.isTTY {
+		return m.title
+	}
+
 	var header string
 	if m.align == "left" {
-		header = m.spinner.View() + str + " " + m.title
+		header = m.spinner.View() + " " + m.title
 	} else {
-		header = str + " " + m.title + " " + m.spinner.View()
+		header = m.title + " " + m.spinner.View()
 	}
-	if !m.showOutput {
-		return header
-	}
-	return header + errbuf.String() + "\n" + outbuf.String()
+	return header + "\n" + out
 }
 
 func (m model) Update(msg tea.Msg) (tea.Model, tea.Cmd) {
 	var cmd tea.Cmd
 	switch msg := msg.(type) {
-	case timeout.TickTimeoutMsg:
-		if msg.TimeoutValue <= 0 {
-			// grab current output before closing for piped instances
-			m.stdout = outbuf.String()
-
-			m.status = exit.StatusAborted
-			return m, tea.Quit
-		}
-		m.timeout = msg.TimeoutValue
-		return m, timeout.Tick(msg.TimeoutValue, msg.Data)
 	case finishCommandMsg:
 		m.stdout = msg.stdout
 		m.stderr = msg.stderr
@@ -146,8 +141,7 @@ func (m model) Update(msg tea.Msg) (tea.Model, tea.Cmd) {
 	case tea.KeyMsg:
 		switch msg.String() {
 		case "ctrl+c":
-			m.aborted = true
-			return m, tea.Quit
+			return m, commandAbort
 		}
 	}
 
